"""

    """

import json
from dataclasses import dataclass
from pathlib import Path

class Conf :
    url = 'url'  # GitHub url of the target repo
    py_ver = 'py_ver'  # python version to use
    module = "module"  # module name to run

@dataclass
class UserRepo :
    user_name: str
    repo_name: str
    user_slash_repo: str
    user_und_repo: str

def get_user_repo_from_url(repo_url) :
    gu = repo_url.split('github.com/')[1]
    user_name = gu.split('/')[0]
    repo_name = gu.split('/')[1]
    user_slash_repo = f'{user_name}/{repo_name}'
    user_und_repo = f'{user_name}_{repo_name}'
    return UserRepo(user_name , repo_name , user_slash_repo , user_und_repo)

def read_json(fp) :
    # if fp is not entered with .json extension, add it
    fp = Path(fp).with_suffix('.json')

    # assume cd is the GitHub dir
    fp = Path.cwd() / 'auto-run-configs' / fp

    with open(fp , 'r') as f :
        return json.load(f)



"""

    """

import subprocess
from pathlib import Path

from .github_release import download_latest_release
from .util import Conf
from .util import get_user_repo_from_url
from .util import read_json

c = Conf()

def make_venv(fp) :
    """ make virtualenv with pyenv, delete existing venv if it exists"""
    j = read_json(fp)

    py_ver = j[c.py_ver]
    url = j[c.url]
    ur = get_user_repo_from_url(url)

    subprocess.run(['pyenv' , 'install' , '--skip-existing' , py_ver])
    subprocess.run(['pyenv' , 'virtualenv-delete' , '-f' , ur.user_und_repo])
    subprocess.run(['pyenv' , 'virtualenv' , py_ver , ur.user_und_repo])

    print(ur.user_und_repo)

def dl_and_ret_dirpath(fp) :
    js = read_json(fp)
    rp_url = js[c.url]
    dirp = download_latest_release(rp_url)
    print(dirp)

def ret_module_2_run_name(fp) :
    js = read_json(fp)
    fbn = js[c.module]
    fn = Path(fbn).with_suffix('.py').name
    print(fn)



"""

    """

import re
import tarfile
from pathlib import Path

import requests

from .util import get_user_repo_from_url

def get_latest_release_tar_url(repo_url) :
    usrp = get_user_repo_from_url(repo_url)
    url = f'https://api.github.com/repos/{usrp.user_slash_repo}/releases/latest'
    r = requests.get(url)
    dct = r.json()
    return dct['tarball_url']

def get_filename_fr_github_resp(r) :
    hdr = r.headers
    cd = hdr['content-disposition']
    pat = 'attachment; filename=(.+)'
    mat = re.findall(pat , cd)
    return mat[0]

def download_latest_release_tarball(repo_url) :
    url = get_latest_release_tar_url(repo_url)

    r = requests.get(url)
    if r.status_code != 200 :
        return

    fn = get_filename_fr_github_resp(r)
    fp = Path.cwd() / fn

    with open(fp , 'wb') as f :
        f.write(r.content)

    return fp

def get_dirname_fr_github_tarball(fp) :
    with tarfile.open(fp) as tar :
        return tar.getnames()[0]

def download_latest_release(repo_url) :
    tar_fp = download_latest_release_tarball(repo_url)

    with tarfile.open(tar_fp) as f :
        f.extractall(tar_fp.parent)

    dirp = tar_fp.parent / get_dirname_fr_github_tarball(tar_fp)
    tar_fp.unlink()

    return dirp
